@@ -93,10 +93,10 @@ object$aic
 BIC.zipath <- function(object, ...)
 object$bic
 
-logLik.zipath <- function(object, newdata, y, weights, na.action=na.pass, 
+logLik.zipath <- function(object, newdata, X, Z, y, offsetx, offsetz, weights, na.action=na.pass, 
                    link = c("logit", "probit", "cloglog", "cauchit", "log"),
 ...){
-if(missing(newdata) || missing(y)) return(object$loglik)
+if(missing(y) || missing(X) || missing(Z)) return(object$loglik)
 linkstr <- match.arg(link)
   linkobj <- make.link(linkstr)
   linkinv <- linkobj$linkinv
@@ -107,16 +107,14 @@ Y <- y
   kz <- NCOL(Z)
     Y0 <- Y <= 0
     Y1 <- Y > 0
-    offsetx <- offsetz <- 0
+#    offsetx <- offsetz <- 0
     ## count mean
     mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
     ## binary mean
     phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
-
     ## log-likelihood for y = 0 and y >= 1
     loglik0 <- log( phi + exp( log(1-phi) - mu ) ) ## -mu = dpois(0, lambda = mu, log = TRUE)
     loglik1 <- log(1-phi) + dpois(Y, lambda = mu, log = TRUE)
-
     ## collect and return
     loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
     loglik
@@ -127,14 +125,13 @@ Y <- y
   kz <- NCOL(Z)
     Y0 <- Y <= 0
     Y1 <- Y > 0
-    offsetx <- offsetz <- 0
+#    offsetx <- offsetz <- 0
     ## count mean
     mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
     ## binary mean
     phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
     ## negbin size
     theta <- exp(parms[(kx+kz)+1])
-
     ## log-likelihood for y = 0 and y >= 1
     loglik0 <- log( phi + exp( log(1-phi) + suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE)) ) )
     loglik1 <- log(1-phi) + suppressWarnings(dnbinom(Y, size = theta, mu = mu, log = TRUE))
@@ -154,10 +151,15 @@ nlambda <- object$nlambda
                       "negbin" = ziNegBin)
 
 ll <- rep(NA, nlambda)
- mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
+if(!is.null(object$terms)){
+mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
     X <- model.matrix(delete.response(object$terms$count), mf, contrasts = object$contrasts$count)
     Z <- model.matrix(delete.response(object$terms$zero),  mf, contrasts = object$contrasts$zero)
-if(missing(weights)) weights <- rep(1, length(y))
+}
+else
+    if(is.null(X) || is.null(Z))
+        stop("X and Z must be provided\n")
+    if(missing(weights)) weights <- rep(1, length(y))
 w <- weights
 if(family == "negbin")
 parms <- rbind(object$coefficients$count, object$coefficients$zero, log(object$theta))
