@@ -80,7 +80,7 @@ compute.2d <- function(y, f, s, family=c("clossR", "closs", "gloss", "qloss")){
         sqrt(2/pi)*u/s^3*exp(-u^2/(2*s^2))
 }
 
-nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, del=1e-10, penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, type.path=c("active", "nonactive", "onestep"), decreasing=FALSE, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, standardize=TRUE, penalty.factor = NULL, maxit=1000, type.init="bst", mstop.init=10, nu.init=0.1, eps=.Machine$double.eps, thresh=1e-6, trace=FALSE){
+nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, reltol=1e-5, penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, type.path=c("active", "nonactive", "onestep"), decreasing=FALSE, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, standardize=TRUE, penalty.factor = NULL, maxit=1000, type.init=c("bst", "ncl", "heu"), mstop.init=10, nu.init=0.1, eps=.Machine$double.eps, epscycle=10, thresh=1e-6, trace=FALSE){
 ### compute h value
     compute.h <- function(rfamily, y, fk_old, s, B){
         if(rfamily=="clossR")
@@ -93,19 +93,22 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     rfamily <- match.arg(rfamily)
     penalty <- match.arg(penalty)
     type.path <- match.arg(type.path)
+    type.init <- match.arg(type.init)
     if(type.path=="active")
 	    active <- 1
     else active <- 0
     if (!is.null(lambda) && length(lambda) > 1 && all(lambda == cummin(lambda))){
 	    decreasing <- TRUE
-	    if(type.path=="active")
-	      warnings("choose type.path='nonactive' with increasing sequence of lambda, or let lambda=rev(lambda) as computed below\n")
-	    lambda <- rev(lambda)
+#	    if(type.path=="active"){
+#	    decreasing <- FALSE
+#	      warnings("choose type.path='nonactive' with increasing sequence of lambda, or let lambda=rev(lambda) as computed below\n")
+#	    lambda <- rev(lambda) 
+#       }
     }
     else if(!is.null(lambda) && length(lambda) > 1 && all(lambda == cummax(lambda)))
 	    decreasing <- FALSE
-    else if(is.null(lambda) && decreasing && type.path=="active")
-	    stop("set decreasing=FALSE or type.path='nonactive'")
+#    else if(is.null(lambda) && decreasing && type.path=="active")
+#	    stop("set decreasing=FALSE or type.path='nonactive'")
     if(rfamily %in% c("closs", "gloss", "qloss"))
         if(!all(names(table(y)) %in% c(1, -1)))
             stop("response variable must be 1/-1 for family ", rfamily, "\n")
@@ -176,7 +179,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
     start <- NULL
     if(is.null(fk) || is.null(lambda)){
         if(type.init %in% c("ncl", "heu")){ ### use ncl function to generate intercept-only model
-            RET <- ncl(y~1, data=data.frame(y, 1), iter=10000, del=1e-20, weights=weights, s=s, rfamily=rfamily, trace=FALSE)
+            RET <- ncl(y~1, data=data.frame(y, 1), iter=10000, reltol=1e-20, weights=weights, s=s, rfamily=rfamily, trace=FALSE)
             if(type.init=="ncl") start <- c(coef(RET), rep(0, nvars))
 ### it is similar to the following optimization results
                                         #fn <- function(b) sum(loss(y, f=b, cost, family = rfamily, s=s))
@@ -217,10 +220,10 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
 ### method A, to obtain lambda values from fitting the penalized regression. 
         lambda <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda.min.ratio=lambda.min.ratio, nlambda=nlambda, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=1, eps=eps, family="gaussian", penalty=penalty)$lambda
 ### with type.init, add two different lambda sequences and make lambda values flexible to have different solution paths
-        if(type.init %in% c("bst", "heu")){
+        #if(type.init %in% c("bst", "heu")){
             if(!decreasing)#{### solution path backward direction
                 lambda <- rev(lambda)
-        }
+        #}
                                         #cat("lambda in method A", lambda[1], "\n")
 ### method B is the same as method A to obtain lmax 
                                         #w <- weights/sum(weights)
@@ -283,7 +286,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
             }
             k <- 1
             d1 <- 10
-            while(d1 > del && k <= iter){
+            while(d1 > reltol && k <= iter){
                 fk_old <- RET$fitted.values
                 h <- compute.h(rfamily, y, fk_old, s, B)
                 if(any(is.nan(h))){ # exit loop
@@ -313,7 +316,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
                 d1 <- sum((fk_old - fk)^2)
                                         #d1 <- sum((fk_old - fk)^2)/sum(fk_old^2) ### this can cause a problem if fk_old is zero
                 if(trace) cat("\n  iteration", k, ": relative change of fk", d1, ", robust loss value", los[k, i], ", penalized loss value", pll[k, i], "\n")
-                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > del && k <= iter: ", (d1 > del && k <= iter), "\n")
+                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
                 k <- k + 1
             }
             if(!stopit){
@@ -340,7 +343,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
             }
             k <- 1
             d1 <- 10
-            while(d1 > del && k <= iter){
+            while(d1 > reltol && k <= iter){
                 fk_old <- RET$fitted.values
                 h <- compute.h(rfamily, y, fk_old, s, B)
 	    	if(any(is.nan(h))){ # exit loop 
@@ -376,6 +379,7 @@ nclreg_fit <- function(x,y, weights, offset=NULL, cost=0.5, rfamily=c("clossR",
 				beta=as.double(matrix(0, ncol=1, nrow=m)),
 				b0=as.double(0),
 				yhat=as.double(rep(0, n)),
+                satu=as.integer(0),
 				PACKAGE="mpath")
      # epsbino, theta are not used in the above Fortran call with family="gaussian"
 #		RET <- glmreg_fit(x=x.act*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda[i],alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor.act, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start.act)
@@ -413,7 +417,7 @@ if(any(is.na(RET$beta))){
                 d1 <- sum((fk_old - fk)^2)
                                         #d1 <- sum((fk_old - fk)^2)/sum(fk_old^2) ### this can cause a problem if fk_old is zero
                 if(trace) cat("\n  iteration", k, ": relative change of fk", d1, ", robust loss value", los[k, i], ", penalized loss value", pll[k, i], "\n")
-                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > del && k <= iter: ", (d1 > del && k <= iter), "\n")
+                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
                 k <- k + 1
             }
             if(!stopit){
@@ -444,6 +448,45 @@ if(any(is.na(RET$beta))){
                     "qloss"=14
                     )
     typeBB <- function(beta, b0){
+        if(type.path=="active" && decreasing)
+	    RET <- .Fortran("nclreg_ad",
+			    x=as.double(x), 
+			    y=as.double(y),
+			    weights=as.double(weights),
+			    n=as.integer(n),
+			    m=as.integer(m),
+			    start=as.double(start), 
+			    etastart=as.double(etastart),
+			    mustart=as.double(mustart),
+			    offset=as.double(offset),
+			    iter=as.integer(iter),
+			    nlambda=as.integer(nlambda), 
+			    lambda=as.double(lambda), 
+			    alpha=as.double(alpha),
+		        gam=as.double(gamma), 
+			    standardize=as.integer(0), 
+			    penaltyfactor=as.double(penalty.factor),
+			    maxit=as.integer(maxit), 
+			    eps=as.double(eps), 
+			    epscycle=as.double(epscycle), 
+			    penalty=as.integer(pentype), 
+			    trace=as.integer(trace), 
+			    del=as.double(reltol), 
+			    rfamily=as.integer(rfamilytype),
+			    B=as.double(B), 
+			    s=as.double(s),
+			    thresh=as.double(thresh),
+			    cost=as.double(cost),
+			    decreasing=as.integer(decreasing),
+			    active=as.integer(active),
+			    beta=as.double(beta),
+		        b0=as.double(b0), 
+			    yhat=as.double(RET$fitted.values), 
+			    los=as.double(rep(0, nlambda)),
+			    pll=as.double(rep(0, nlambda)),
+			    nlambdacal=as.integer(0),
+			    PACKAGE="mpath")
+    else
 	    RET <- .Fortran("nclreg_fortran",
 			    x=as.double(x), 
 			    y=as.double(y),
@@ -454,35 +497,34 @@ if(any(is.na(RET$beta))){
 			    etastart=as.double(etastart),
 			    mustart=as.double(mustart),
 			    offset=as.double(offset),
+			    iter=as.integer(iter),
 			    nlambda=as.integer(nlambda), 
 			    lambda=as.double(lambda), 
 			    alpha=as.double(alpha),
-		            gam=as.double(gamma), 
+		        gam=as.double(gamma), 
 			    standardize=as.integer(0), 
 			    penaltyfactor=as.double(penalty.factor),
 			    maxit=as.integer(maxit), 
 			    eps=as.double(eps), 
-			    family=as.integer(1), 
+			    epscycle=as.double(epscycle), 
 			    penalty=as.integer(pentype), 
 			    trace=as.integer(trace), 
-			    beta=as.double(beta),
-		            b0=as.double(b0), 
-			    yhat=as.double(RET$fitted.values), 
-			    iter=as.integer(iter),
-			    del=as.double(del), 
+			    del=as.double(reltol), 
 			    rfamily=as.integer(rfamilytype),
 			    B=as.double(B), 
 			    s=as.double(s),
+			    thresh=as.double(thresh),
+			    cost=as.double(cost),
+			    decreasing=as.integer(decreasing),
+			    active=as.integer(active),
+			    beta=as.double(beta),
+		        b0=as.double(b0), 
+			    yhat=as.double(RET$fitted.values), 
 			    los=as.double(rep(0, nlambda)),
 			    pll=as.double(rep(0, nlambda)),
-			    rescale=as.integer(0),
-			    thresh=as.double(thresh),
-			    epsbino=as.double(0),
-			    theta=as.double(0),
-			    cost=as.double(cost),
-			    active=as.integer(active),
+			    nlambdacal=as.integer(0),
 			    PACKAGE="mpath")
-       	    list(beta=matrix(RET$beta, ncol=nlambda), b0=RET$b0, RET=RET, risk=RET$los, pll=RET$pll)
+       	    list(beta=matrix(RET$beta, ncol=nlambda), b0=RET$b0, RET=RET, risk=RET$los, pll=RET$pll, nlambdacal=RET$nlambdacal)
     }
 ### update for one element of lambda depending on increasing sequence of lambda (last element of lambda) or decreasing sequence of lambda (then first element of lambda) in each MM iteration, and iterate until convergency of prediction. Then fit a solution path based on the sequence of lambda. Experiment code. Argument direction has been removed.
     typeC <- function(beta, b0){
@@ -494,7 +536,7 @@ if(any(is.na(RET$beta))){
         fk <- RET$fitted.values
         d1 <- 10
         lam <- lambda[ifelse(decreasing, nlambda, 1)]
-	while(d1 > del && k <= iter){
+	while(d1 > reltol && k <= iter){
 	    fk_old <- fk
             h <- compute.h(rfamily, y, fk_old, s, B)
 	    RET <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lam, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start)
@@ -503,7 +545,7 @@ if(any(is.na(RET$beta))){
             start <- coef(RET)
             d1 <- sum((fk_old - fk)^2)
             if(trace) cat("\n  iteration", k, ": relative change of fk", d1, "\n")
-            if(trace) cat("  d1=", d1, ", k=", k, ", d1 > del && k <= iter: ", (d1 > del && k <= iter), "\n")
+            if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
             k <- k + 1
         }
 ### fit a solution path    
@@ -558,6 +600,7 @@ if(any(is.na(RET$beta))){
     RET$s <- s
     RET$risk <- tmp$risk
     RET$pll <- tmp$pll
+    RET$nlambdacal <- tmp$nlambdacal
     RET$rfamily <- RET$family <- rfamily
     RET$type.init <- type.init
     RET$mstop.init <- mstop.init
